/*  ----------------------------------------------------------------<Prolog>-
    Name:       com.imatix.ServletRunner.HttpSessionImpl
    Title:      Implementation of javax.servlet.http.HttpSession
    Package:    iMatix Servlet Runner

    Written:    99/03/15  Ewen McNeill <ewen@imatix.com>
    Revised:    99/03/15

    Copyright:  Copyright (c) 1991-99 iMatix Corporation
    License:    This is free software; you can redistribute it and/or modify
                it under the terms of the SMT License Agreement as provided
                in the file LICENSE.TXT.  This software is distributed in
                the hope that it will be useful, but without any warranty.
 ------------------------------------------------------------------</Prolog>-*/

package com.imatix.ServletRunner;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;
import javax.servlet.http.*;

/**
    iMatix Servlet Runner implementation of 
    javax.servlet.http.HttpSession
 */
public class HttpSessionImpl implements HttpSession
{
   private long                    creationTime;
   private long                    lastAccessTime;
   private Hashtable               sessionValues;
   private boolean                 newSession;
   private boolean                 sessionValid;
   private String                  sessionId;
   private HttpSessionContextImpl  sessionContext;

   private static int              sessionSequence = 0;

   /** 
       Constructor -- creates a new, empty, session
     */
   protected HttpSessionImpl (HttpSessionContextImpl context)
   {
      creationTime   = System.currentTimeMillis();
      lastAccessTime = 0;

      sessionValues  = new Hashtable();
      newSession     = true;
      sessionValid   = true;
      sessionContext = context;

      // Create session ID.  We need a string that will be unique to
      // this client, and unique to this servlet. To do this we generate
      // the session ID from the current time plus the sequence number.
      sessionId      = "sess" +
                       Long.toString(System.currentTimeMillis()) + "s" +
		       Integer.toString(++sessionSequence);
   }

   /**
      Record session access (by client).
     */
   protected void recordSessionAccess()
   {
      lastAccessTime = System.currentTimeMillis();
      newSession     = false;
   }

   /**
      Validate session -- throw IllegalStateException if not valid
     */
   private void validateSession() throws IllegalStateException
   {
       if (! sessionValid)
           throw new 
	       IllegalStateException("Session " + sessionId + " is invalid");
   }

   /** 
       Returns ID assigned to this session.
     */
   public String getId () throws IllegalStateException
   {
       validateSession();
       return sessionId;
   }

   /**
       Returns the context in which this session is bound.
     */
   public HttpSessionContext getSessionContext () throws IllegalStateException
   {
       validateSession();
       return sessionContext;
   }

   /**
       Returns time when this session was created.
     */
   public long getCreationTime () throws IllegalStateException
   {
       validateSession();
       return creationTime;
   }

   /**
       Returns time when this session was last acccessed by a client.
     */
   public long getLastAccessedTime () throws IllegalStateException
   {
       validateSession();
       return lastAccessTime;
   }

   /**
       Invalidates the session and removes it from the session context.
     */
   public void invalidate ()
   {
       sessionValid = false;
   }

   /**
       Stores value in the session.
     */
   public void putValue (String name, Object value) throws IllegalStateException
   {
       validateSession();
       sessionValues.put(name, value);
       if (value != null && value instanceof HttpSessionBindingListener)
           ((HttpSessionBindingListener)value).valueBound(
	           new HttpSessionBindingEvent(this, name));
   }

   /**
       Retrieves value from the session.
     */
   public Object getValue (String name) throws IllegalStateException
   {
       validateSession();
       return sessionValues.get(name);
   }

   /**
       Removes value from the session.
     */
   public void removeValue (String name) throws IllegalStateException
   {
       validateSession();

       Object value = getValue(name);
       sessionValues.remove(name);

       if (value != null && value instanceof HttpSessionBindingListener)
           ((HttpSessionBindingListener)value).valueUnbound(
	           new HttpSessionBindingEvent(this, name));
   }
   
   /**
       Retrieve names of values stored in the session.
     */
   public String [] getValueNames () throws IllegalStateException
   {
       validateSession();

       Enumeration valEnum    = sessionValues.keys();
       Vector      valueNames = new Vector();

       while (valEnum.hasMoreElements())
       {
           valueNames.addElement(valEnum.nextElement());
       }

       String valueNameArray[] = new String[valueNames.size()];
       valueNames.copyInto(valueNameArray);
       return valueNameArray;
   }

   /**
       Returns true if the session is "new".  A session is considered
       new if it has just been created, or if the client has not yet
       joined the session (by sending in a request with the session
       identifier in it).
     */
   public boolean isNew () throws IllegalStateException
   {
       validateSession();
       return newSession;
   }
}
