/*  ----------------------------------------------------------------<Prolog>-
    Name:       com.imatix.ServletRunner.ServletContextImpl
    Title:      Implementation of javax.servlet.ServletContext
    Package:    iMatix Servlet Runner

    Written:    99/01/11  Ewen McNeill <ewen@imatix.com>
    Revised:    99/01/11

    Copyright:  Copyright (c) 1991-99 iMatix Corporation
    License:    This is free software; you can redistribute it and/or modify
                it under the terms of the SMT License Agreement as provided
                in the file LICENSE.TXT.  This software is distributed in
                the hope that it will be useful, but without any warranty.
 ------------------------------------------------------------------</Prolog>-*/

package com.imatix.ServletRunner;

import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

/**
    iMatix Servlet Runner implementation of javax.servlet.ServletContext

    This implementation has stub functions for most of the required
    functions but additional functionality may be added as required.
 */
public class ServletContextImpl implements ServletContext
{
    ServletController controller;
    PrintWriter       logfile;

    /**
        Constructor.  Saves the parent class away for use in call backs.
     */
    ServletContextImpl(ServletController parent, PrintWriter log)
    {
        controller = parent;
	logfile    = log;
    }

    /**
        Returns the servlet of the specified name, or null if not 
        found.  When the servlet is returned it is initialized and
        ready to accept service requests.
     */
    public Servlet getServlet(String name)
    {
        return (Servlet)controller.getServletsTable().get(name);
    }

    /**
        Returns an enumeration of Servlet objects in this server.
	Only servlets in the same namespace (ie, that are accessible)
	will be returned.  The servlet calling this method will be
	included.

	@deprecated
     */
    public Enumeration getServlets()
    {
        return controller.getServletsTable().elements();
    }

    /**
        Returns an enumeration of Servlet names in this server.
	Only servlets in the same namespace (ie, that are accessible)
	will be returned.  The servlet calling this method will be
	included.
     */
    public Enumeration getServletNames()
    {
        return controller.getServletsTable().keys();
    }

    /**
        Write an entry to the servlet log file.
     */
    public void log(String msg)
    {
        controller.servletLog(msg);
    }
    
    /**
        Write an entry and an associated stack traceback to the servlet
	log file.
     */
    public void log(Exception exception, String msg)
    {
        synchronized (logfile)
	{
           controller.servletLog("Servlet exception: " + msg);
           exception.printStackTrace(logfile);
	}
    }

    /**
        Translate a virtual path into the real path (on disk).
     */
    public String getRealPath(String path)
    {
        return controller.translatePath(path);
    }

    /**
        Get the mime type for the specified file, or null if not known.
     */
    public String getMimeType(String file)
    {
        return controller.determineMimeType(file);
    }

    /**
        Get the name and version of the network service under which
	the servlet is running.  

	This returns the same value as the CGI variable SERVER_SOFTWARE.
	This value is set on a per-invocation process; although it shouldn't
	change from invocation to invocation while the ServletRunner is
	running.
     */
    public String getServerInfo()
    {
        return controller.getServerInfo();
    }

    /**
        Returns the value of the named attribute of the network service,
	or null if the attribute does not exist.

	XXX: Not implemented.  Always returns null.  Not entirely sure
	where "named attribute of the network service" should be
	retrieved from.
     */
    public Object getAttribute(String name)
    {
        return null;
    }
}
