/*  ----------------------------------------------------------------<Prolog>-
    Name:       com.imatix.ServletRunner.ServletController
    Title:      Interface: functions required for call back to Servlet Control
    Package:    iMatix Servlet Runner

    Written:    99/01/19  Ewen McNeill <ewen@imatix.com>
    Revised:    99/01/19

    Copyright:  Copyright (c) 1991-99 iMatix Corporation
    License:    This is free software; you can redistribute it and/or modify
                it under the terms of the SMT License Agreement as provided
                in the file LICENSE.TXT.  This software is distributed in
                the hope that it will be useful, but without any warranty.
 ------------------------------------------------------------------</Prolog>-*/

package com.imatix.ServletRunner;

import java.util.Dictionary;

/**
    This interface enumerates the functions required by a Servlet
    Controller to allow the context, etc, to call back into it to 
    have actions performed.

    The methods are intended to be used by the Servlet Runner only; but
    apparently private/protected interface methods are not allowed.  Not
    sure why that is.
 */
public interface ServletController 
{
    /**
        Returns a reference to the internal directory of loaded 
	Servlets, which can be used to determine which other Servlets
	are running.  Contacting those servlets directly is NOT 
	recommended.  Access to the table should be synchronized against
	the returned table.
     */
    public Dictionary getServletsTable();

    /** 
        Translate a given path from a URI/virtual path into a path
	on the disk.
     */
    public String translatePath(String virtualPath);

    /**
        Determine the mime type to be used with the file (if possible).
	Returns null if the mime type cannot be determined.
     */
    public String determineMimeType(String fileName);

    /**
        Gets the type of server in use.  This returns the static
	identification of the servlet runner.
     */
    public String getServerInfo();

    /** 
        Write an entry into the log file, with the servlet name
	and time and date stamp.
     */
    public void servletLog(String message);
}
